<?php
// $Id: fast_gallery.admin.inc, 2008/10/03 ncy $

/**
 * @file
 * Administrative page callbacks for the Fast Gallery module.
 * 
 * @author Raphael Schr - www.schaerwebdesign.ch
 * @author Nicholas C. Yang - www.nyanginator.com
 */

/**
 * Default Fast Gallery settings page.
 */
function fast_gallery_settings_page($op = NULL) {
  $output = drupal_get_form('fast_gallery_general_settings_form');
  return $output;
}

/**
 * Callback function for admin settings
 */
function fast_gallery_general_settings_form() {

  $form['fg_setup'] = array(
    '#type' => 'fieldset',
    '#title' => t('Setup'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_setup']['fast_gallery_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path to Gallery'),
    '#default_value' => variable_get('fast_gallery_path', ''),
    '#description' => t("Path to images from which albums should be created. Relative to site's root."),
  );

  $form['fg_setup']['fast_gallery_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Gallery Title'),
    '#default_value' => variable_get('fast_gallery_title', 'Fast Gallery'),
    '#description' => t('Title to display at the top of the gallery.'),
    '#size' => 30,
  );

  $form['fg_setup']['fast_gallery_path_alias'] = array(
    '#type' => 'textfield',
    '#title' => t('Fast Gallery Path Alias'),
    '#default_value' => variable_get('fast_gallery_path_alias', 'fast_gallery'),
    '#description' => t('Custom set path alias for the gallery.'),
    '#size' => 30,
  );

  $form['fg_appearance'] = array(
    '#type' => 'fieldset',
    '#title' => t('Appearance'),
    '#description' => t('Customize how images and captions are displayed.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_appearance']['fast_gallery_nr_per_page'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of images per page'),
    '#default_value' => variable_get('fast_gallery_nr_per_page', 8),
    '#description' => t('Number of images to display on each page.'),
    '#size' => 2,
  );

  $form['fg_appearance']['fast_gallery_row_count'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of images per row'),
    '#default_value' => variable_get('fast_gallery_row_count', 4),
    '#description' => t('Number of images to display on each row. Set this to more than the maximum number of thumbnails that will fit across the page.'),
    '#size' => 2,
  );

  $form['fg_appearance']['fast_gallery_teaser_length'] = array(
    '#type' => 'textfield',
    '#title' => t('Teaser length of captions and folder labels'),
    '#default_value' => variable_get('fast_gallery_teaser_length', 8),
    '#description' => t('Maximum number of words in captions and folder labels'),
    '#size' => 2,
  );

  $option = array(
    'none' => t('<none>'),
    'desc' => t('Description'),
    'date' => t('Date Taken'),
    'name' => t('Filename'),
    'xptitle' => t('Windows XP Title'),
    'xpcomments' => t('Windows XP Comments'),
    'xpkeywords' => t('Windows XP Keywords'),
  );
  $form['fg_appearance']['fast_gallery_caption_display_1'] = array(
    '#type' => 'select',
    '#title' => t('Image caption display #1'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_caption_display_1', 'desc'),
  );
  $form['fg_appearance']['fast_gallery_caption_display_2'] = array(
    '#type' => 'select',
    '#title' => t('Image caption display #2'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_caption_display_2', '<none>'),
  );
  $form['fg_appearance']['fast_gallery_caption_display_3'] = array(
    '#type' => 'select',
    '#title' => t('Image caption display #3'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_caption_display_3', '<none>'),
  );

  $form['fg_appearance']['fast_gallery_hierarchy'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hierarchy'),
    '#default_value' => variable_get('fast_gallery_hierarchy', 1),
    '#description' => t('Choose if you want to display images in a gallery with folders as opposed to a single flat gallery with no folders.'),
  );

  $form['fg_appearance']['fast_gallery_display_folder'] = array(
    '#type' => 'checkbox',
    '#title' => t('Folders as Text Links'),
    '#default_value' => variable_get('fast_gallery_display_folder', 0),
    '#description' => t('Choose if you want to display folders as text links.'),
  );

  $js_framework_options = array();
  $js_framework_options[0] = t('<none>');
  $js_framework_default = 0;
  if (module_exists('thickbox'))
    $js_framework_options[1] = t('Thickbox');
  if (module_exists('lightbox2')) {
    $js_framework_default = 2;
    $js_framework_options[2] = t('Lightbox2 (no slideshow)');
    $js_framework_options[3] = t('Lightbox2 (with slideshow)');
  }

  $form['fg_appearance']['fast_gallery_js_framework'] = array(
    '#type' => 'select',
    '#title' => t('Choose your Javascript framework'),
    '#options' => $js_framework_options,
    '#default_value' => variable_get('fast_gallery_js_framework', $js_framework_default),
    '#description' => t('Thickbox and Lightbox are the two options for displaying images.'),
  );
 
  $ic_link = l('Imagecache', 'admin/build/imagecache');
  $form['fast_gallery_operations'] = array(
    '#type' => 'fieldset',
    '#title' => t('Some operations'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#tree' => FALSE,
    '#description' => t('Clear the database whenever you change the gallery path. Rescan the database whenever you add images, delete images, move images, or restructure the hierarchy. Rebuilding the database can take some time, so use only when necessary. Flush thumbs to delete and force recaching of thumbnails. If you are using Imagecache, you need to flush the thumbnails on the Imagecache settings page: ') . $ic_link,
  );

  $form['fast_gallery_operations']['fast_gallery_clear'] = array(
    '#type' => 'button',
    '#value' => t('Clear DB'),
    '#ahah' => array(
      'path' => 'fast_gallery/clear',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Clearing DB')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  $form['fast_gallery_operations']['fast_gallery_rescan'] = array(
    '#type' => 'button',
    '#value' => t('Rescan'),
    '#ahah' => array(
      'path' => 'fast_gallery/rescan',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Scan for new pictures...')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  $form['fast_gallery_operations']['fast_gallery_flush_thumbs'] = array(
    '#type' => 'button',
    '#value' => t('Flush Internal Cache'),
    '#ahah' => array(
      'path' => 'fast_gallery/flush',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Flushing thumbnails')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Settings'),
  );

  return $form;
}

/**
 * Validation function for Fast Gallery General Settings form.
 */
function fast_gallery_general_settings_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['fast_gallery_nr_per_page'])) {
    form_error($form['fg_appearance']['fast_gallery_nr_per_page'], t('Number of images per page has to be numeric.'));
  }

  if (!is_numeric($form_state['values']['fast_gallery_row_count'])) {
    form_error($form['fg_appearance']['fast_gallery_row_count'], t('Number of images per row has to be numeric.'));
  }

  if (!is_numeric($form_state['values']['fast_gallery_teaser_length'])) {
    form_error($form['fg_appearance']['fast_gallery_teaser_length'], t('Teaser length has to be numeric.'));
  }

  $path = $form_state['values']['fast_gallery_path'];
  if (!file_exists($path)) {
    form_error($form['fg_setup']['fast_gallery_path'], t('This folder doesn\'t exist.'));
  }
}

/**
 * Process the submitted general settings form. Copied mostly from
 * system_settings_form_submit(). Need to also do additional processing and
 * changes to database for any changes in title and alias.
 */
function fast_gallery_general_settings_form_submit($form, &$form_state) {
  // Check if the alias was changed and update database entries if so
  $current_route = variable_get('fast_gallery_path_alias', 'fast_gallery');
  $new_route = $form_state['values']['fast_gallery_path_alias'];
  if ($current_route != $new_route) {
    $sql = "UPDATE {menu_router} SET path='%s', tab_root='%s' WHERE page_callback='fast_gallery_page'";
    db_query($sql, $new_route, $new_route);
    $sql = "UPDATE {menu_links} SET link_path='%s', router_path='%s' WHERE
      link_path='%s'";
    db_query($sql, $new_route, $new_route, $current_route);
  }

  // Check if the title was changed and update database entries if so
  $current_title = variable_get('fast_gallery_title', 'Fast Gallery');
  $new_title = $form_state['values']['fast_gallery_title'];
  if ($current_title != $new_title) {
    $sql = "UPDATE {menu_router} SET title='%s' WHERE page_callback='fast_gallery_page'";
    db_query($sql, $new_title);
    $sql = "UPDATE {menu_links} SET link_title='%s' WHERE link_path='%s'";
    db_query($sql, $new_title, $new_route);
  }

  // Now process and save the form settings

  // Exclude unnecessary elements
  unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

  // Update variables
  foreach ($form_state['values'] as $key => $value) {
    if (is_array($value) && isset($form_state['values']['array_filter'])) {
      $value = array_keys(array_filter($value));
    }
    variable_set($key, $value);
  }

  drupal_set_message(t('The configuration options have been saved.'));

  menu_cache_clear_all();
  cache_clear_all();
  drupal_rebuild_theme_registry();
}

/**
 * Advanced Settings form
 */
function fast_gallery_advanced_settings_form() {
  if (module_exists('imagecache')) {
    $arICPresetts = imagecache_presets();
    $optionPresets = array();
    foreach ($arICPresetts as $value) {
      $optionPresets[$value['presetname']] = $value['presetname'];
    }
    $optionPresetsBig = $optionPresets;
  }
  $optionPresets['<none>'] = '<none>';
  $optionPresetsBig['<none>'] = '<none>';

  $ic_link = l('Imagecache', 'admin/build/imagecache');
  $form['ic'] = array(
    '#type' => 'fieldset',
    '#title' => t('ImageCache'),
    '#description' => t(''),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('If you install Imagecache (IC) Presets, be aware that this function is not bulletproof. If you hit this button multiple times you will get multiple presets with the same name! When you uninstall Fast Gallery, Imagecache presets are left intact. Please be sure to check the Imagecache preset page to see what presets are installed: ') . $ic_link,
  );

  $form['ic']['fast_gallery_presets_install'] = array(
    '#type' => 'button',
    '#value' => t('Install IC Presets'),
    '#ahah' => array(
      'path' => 'fast_gallery/presets_install',
      'method' => 'replace',
      'effect' => 'fade',
      'progress' => array(
        'type' => 'bar',
        'message' => t('Installing Imagecache presets...')
      ),
      'wrapper' => 'attach-wrapper',
    )
  );

  $form['ic']['fast_gallery_ic'] = array(
    '#type' => 'select',
    '#title' => t('Fill in the Imagecache preset name for thumbnails'),
    '#options' => $optionPresets,
    '#default_value' => variable_get('fast_gallery_ic', '<none>'),
    '#description' => t('Choose Imagecache preset that should be applied to image thumbnails.'),
  );

  $form['ic']['fast_gallery_ic_big'] = array(
    '#type' => 'select',
    '#title' => t('Fill in the Imagecache preset name for full-size display'),
    '#default_value' => variable_get('fast_gallery_ic_big', '<none>'),
    '#options' => $optionPresetsBig,
    '#description' => t('Choose Imagecache preset that should be applied to full-size images. Use &lt;none&gt; if you just want to use the original image.'),
  );

  $optionSorting = array(
    'asc' => t('Ascending'),
    'desc' => t('Descending')
  );
  
  $option = array(
    'path' => t('Name'),
    'date' => t('Date taken'),
    'filec'=> t('File created'),
    'rand' => t('Random'),
  );

  $form['fg_image_sorting'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sorting Images'),
    '#description' => t('Various image sorting options.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_image_sorting']['fast_gallery_sort'] = array(
    '#type' => 'select',
    '#title' => t('Sort by'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_sort', 'date'),
  );

  $form['fg_image_sorting']['fast_gallery_direction'] = array(
    '#type' => 'select',
    '#title' => t('Ascending/Descending'),
    '#options' => $optionSorting,
    '#default_value' => variable_get('fast_gallery_direction', 'desc'),
  );
  
  $form['fg_folder_sorting'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sorting Folders'),
    '#description' => t('The first picture of the folder is taken as sorting criteria for options that require a picture.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_folder_sorting']['fast_gallery_sort_folder'] = array(
    '#type' => 'select',
    '#title' => t('Sort by'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_sort_folder', 'date'),
  );

  $form['fg_folder_sorting']['fast_gallery_folder_direction'] = array(
    '#type' => 'select',
    '#title' => t('Ascending/Descending'),
    '#options' => $optionSorting,
    '#default_value' => variable_get('fast_gallery_folder_direction', 'desc'),
  );

  $form['fg_flat_sorting'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sorting for Hierarchy Disabled'),
    '#description' => t('When Hierarchy is disabled, this sorting option will be used.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_flat_sorting']['fast_gallery_sort_flat'] = array(
    '#type' => 'select',
    '#title' => t('Sort by'),
    '#options' => $option,
    '#default_value' => variable_get('fast_gallery_sort_flat', 'date'),
  );

  $form['fg_flat_sorting']['fast_gallery_flat_direction'] = array(
    '#type' => 'select',
    '#title' => t('Ascending/Descending'),
    '#options' => $optionSorting,
    '#default_value' => variable_get('fast_gallery_flat_direction', 'desc'),
  );
  
  $options = array(
    0 => t('Manual'),
    1 => t('Automatic')
  );

  $form['fg_update'] = array(
    '#type' => 'fieldset',
    '#title' => t('Updating'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['fg_update']['fast_gallery_update_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Update mode'),
    '#options' => $options,
    '#default_value' => variable_get('fast_gallery_update_mode', 0),
    '#description' => t('Choose automatic to update the gallery with every cron job. Choose manual to update by clicking on Rescan on the General settings page.'),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Settings'),
  );

  return $form;
}

function fast_gallery_advanced_settings_form_submit($form, &$form_state) {
  // Exclude unnecessary elements
  unset($form_state['values']['submit'], $form_state['values']['reset'], $form_state['values']['form_id'], $form_state['values']['op'], $form_state['values']['form_token'], $form_state['values']['form_build_id']);

  // Update variables
  foreach ($form_state['values'] as $key => $value) {
    if (is_array($value) && isset($form_state['values']['array_filter'])) {
      $value = array_keys(array_filter($value));
    }
    variable_set($key, $value);
  }

  drupal_set_message(t('The configuration options have been saved.'));

  cache_clear_all();
  drupal_rebuild_theme_registry();
}
