<?php
// $Id: fast_gallery.cache.class.php, 2008/10/03 ncy $

/**
 * @file
 * This class provides a very simple cache for the thumbnails, in case
 * Imagecache is not being used (or can't be used).
 *
 * @author Raphael Schr - www.schaerwebdesign.ch
 * @author Nicholas C. Yang - www.nyanginator.com
 */

class FastGalleryCache {
  static private $instance = null;
  
  /**
   * We are implementing a singleton pattern
   */
  private function __construct() {
  }
  
  public function getInstance() {
    if(is_null(self::$instance)) {
      self::$instance = new self;
    }
    return self::$instance;
  }
  
  /**
   * Creates a thumb and copies it into the same folder with .thumb extension.
   *
   * @param name
   *   Name of the original image.
   * @param width
   *   Desired width of the thumbnail.
   * @param height
   *   Desired height of the thumbnail.
   */
  public function createthumb($name, $width, $height) {
    // Return without doing anything if the thumbnail already exists
    if (file_exists($name . '.thumb'))
      return true;

    // Get the file extension
    $arTmp = explode('.',$name);
    $type = strtolower(end($arTmp));

    // We're only supporting JPG, PNG, GIF
    if ($type == 'jpg' || $type == 'jpeg')
      $src_img = imagecreatefromjpeg($name);
    else if ($type == 'png')
      $src_img = imagecreatefrompng($name);
    else if ($type == 'gif')
      $src_img = imagecreatefromgif($name);
    else {
      drupal_set_message('Internal cache does not support files of type: .'.$type);
      return;
    }

    // New dimensions
    $width_orig = imagesx($src_img);
    $height_orig = imagesy($src_img);
    $ratio_orig = $width_orig/$height_orig;

    if ($width/$height > $ratio_orig) {
      $width = $height*$ratio_orig;
    }
    else {
      $height = $width/$ratio_orig;
    }

    // Resample
    $dst_img = imagecreatetruecolor($width, $height);
    imagecopyresampled($dst_img, $src_img, 0, 0, 0, 0, $width, $height, $width_orig, $height_orig);

    // Save out the thumbnail file based on its type
    if ($type == 'jpg' || $type == 'jpeg')
      imagejpeg($dst_img, $name . '.thumb');
    else if ($type == 'png')
      imagepng($dst_img, $name. '.thumb');
    else if ($type == 'gif')
      imagegif($dst_img, $name. '.thumb');
    else if ($type == 'bmp')
      imagewbmp($dst_img, $name . '.thumb');

    // Some cleanup
    imagedestroy($dst_img);
    imagedestroy($src_img);
  }

  /**
   * Removes all the thumbs.
   *
   * @param path
   *   Path of directory where thumbs are located.
   * @param recursive
   *   Specify whether to recurse through subdirectories.
   */
  public function flushThumbs($path = '', $recursive = FALSE){
    // Get all .thumb files from given path
    $files = glob($path.'*.thumb');

    // Delete the thumbs in this path, skip over directories (arrays)
    foreach ($files as $img) {
      if (!is_array($img))
        unlink($img);
    }

    // Recurse through subdirectories if necessary
    if ($recursive) {
      $dirs = glob($path.'*', GLOB_MARK | GLOB_ONLYDIR);
      foreach ($dirs as $dir) {
        $this->flushThumbs($dir, TRUE);
      }
    }
  }
}
